import pytest

from zndraw import ZnDraw


def test_selection(server, s22):
    """Test the server fixture."""
    vis = ZnDraw(url=server, token="test_token")
    vis.extend(s22)

    assert len(vis.atoms) == 8

    with pytest.raises(ValueError):
        vis.selection = ["Hello"]

    with pytest.raises(IndexError):
        vis.selection = [0, 1, 2, 25]

    with pytest.raises(IndexError):
        vis.selection = [0, 1, 2, -10]

    vis.selection = [0, 7, 6, 5, 4]
    assert vis.selection == [0, 7, 6, 5, 4]

    with pytest.raises(ValueError):
        vis.selection = [0, 0, 2]

    with pytest.raises(ValueError):
        vis.selection = "Hello"



# Run tests with and without eventlet
import eventlet

eventlet.monkey_patch()

import datetime  # noqa
import uuid
import os

import tqdm
from rdkit2ase import smiles2conformers

#### Import ZnDraw ####
from zndraw import ZnDraw

vis = ZnDraw(url=os.environ["ZNDRAW_URL"], token=uuid.uuid4().hex)
#### ------------- ####

# vis._refresh_client.delay_between_calls = datetime.timedelta(milliseconds=10)

conformers = smiles2conformers("CCCCCCCCCO", numConfs=1000)

# append
for atoms in tqdm.tqdm(conformers, desc="append", ncols=80):
    vis.append(atoms)

# read
for i in tqdm.trange(len(vis), desc="getitem", ncols=80):
    _ = vis[i]

# delete
for i in tqdm.tqdm(range(len(vis) - 1, -1, -1), desc="delete", ncols=80):
    del vis[i]

# extend
vis.extend(conformers)

# read_all
print(f"len(vis[:]): {len(vis[:])}")



import pytest
import znsocket
from ase.build import molecule

from zndraw import Box, Sphere, ZnDraw


def test_geometries(server, s22):
    """Test the server fixture."""
    vis = ZnDraw(url=server, token="test_token")
    vis.extend(s22)

    assert vis.geometries == []
    vis.geometries = [Box(position=[1, 2, 3]), Sphere()]
    vis.socket.sleep(0.1)

    assert len(vis.geometries) == 2
    assert isinstance(vis.geometries[0], Box)
    assert isinstance(vis.geometries[1], Sphere)

    assert vis.geometries[0].position == [1, 2, 3]
    assert vis.geometries[1].position == [0, 0, 0]

    with pytest.raises(ValueError):
        vis.geometries = ["Geometries are not string!"]

    with pytest.raises(ValueError):
        vis.geometries = "Geometries are not string!"


def test_geometry_selection_position(server):
    vis = ZnDraw(url=server, token="test_token")
    vis.append(molecule("CH4"))
    assert len(vis.geometries) == 0
    geometry_queue = znsocket.Dict(
        r=vis.r,
        socket=vis._refresh_client,
        key=f"queue:{vis.token}:geometry",
    )
    geometry_queue["Plane"] = {
        "material": {
            "color": "#62929e",
            "opacity": 0.2,
            "wireframe": False,
            "outlines": False,
        },
        "width": 10,
        "height": 10,
    }
    vis.socket.emit("room:worker:run")
    vis.socket.sleep(5)

    assert len(vis.geometries) == 1
    assert vis.geometries[0].position == [0, 0, 0]

    # now with a selection
    vis.selection = [1]
    geometry_queue["Plane"] = {
        "material": {
            "color": "#62929e",
            "opacity": 0.2,
            "wireframe": False,
            "outlines": False,
        },
        "width": 10,
        "height": 10,
    }
    vis.socket.emit("room:worker:run")
    vis.socket.sleep(5)

    assert len(vis.geometries) == 2
    assert vis.geometries[1].position == vis.atoms.positions[1].tolist()
    assert vis.geometries[1].position != [0, 0, 0]

    # now with a selection of multiple atoms
    vis.selection = [1, 2]
    geometry_queue["Plane"] = {
        "material": {
            "color": "#62929e",
            "opacity": 0.2,
            "wireframe": False,
            "outlines": False,
        },
        "width": 10,
        "height": 10,
    }
    vis.socket.emit("room:worker:run")
    vis.socket.sleep(5)

    assert len(vis.geometries) == 3
    assert (
        vis.geometries[2].position
        == vis.atoms.get_center_of_mass(indices=[1, 2]).tolist()
    )



import ase

from zndraw.tasks import FileIO, get_generator_from_filename


def test_get_generator_from_filename():
    file = FileIO(
        name="https://raw.githubusercontent.com/LarsSchaaf/Guaranteed-Non-Local-Molecular-Dataset/main/gnl-dataset/GNL-v0.2/gnl-v0.2-test.xyz"
    )
    generator = get_generator_from_filename(file)

    assert isinstance(next(iter(generator)), ase.Atoms)



import subprocess

import typer

app = typer.Typer()


@app.command()
def main(file: str, n: int, browser: bool = False):
    cmd = ["zndraw", file]
    if not browser:
        cmd.append("--no-browser")
    # run cmd n times in parallel
    for _ in range(n):
        subprocess.Popen(cmd)


if __name__ == "__main__":
    app()



import numpy as np
import numpy.testing as npt

from zndraw.utils import (
    convert_url_to_http,
    direction_to_euler,
    euler_to_direction,
    parse_url,
)


def test_parse_url():
    assert parse_url("http://example.com") == ("http://example.com", None)
    assert parse_url("http://example.com/path") == ("http://example.com", "path")
    assert parse_url("http://localhost:5000") == ("http://localhost:5000", None)
    assert parse_url("http://localhost:5000/path") == ("http://localhost:5000", "path")
    assert parse_url("http://localhost:5000/path/") == ("http://localhost:5000", "path")


def test_conversion_utils():
    """Test conversion functions"""
    direction = np.array([1, 2, 3])
    direction = direction / np.linalg.norm(direction)
    euler = direction_to_euler(direction)
    new_direction = euler_to_direction(euler)

    npt.assert_allclose(direction, new_direction, atol=1e-6)

    direction = np.array([1, 0, 0])
    euler = direction_to_euler(direction)
    new_direction = euler_to_direction(euler)

    npt.assert_allclose(direction, new_direction, atol=1e-6)
    npt.assert_allclose(euler, [0, 0, 0], atol=1e-6)

    direction = np.array([0, -1, 0])
    euler = direction_to_euler(direction)
    new_direction = euler_to_direction(euler)

    npt.assert_allclose(direction, new_direction, atol=1e-6)


def test_url():
    # safe url
    before_url = "ws://localhost:8000/token/1234"
    url = convert_url_to_http(before_url)
    assert url == "http://localhost:8000/token/1234"

    # unsafe url containing ws in token
    before_url = "ws://localhost:8000/token/eNwsdW5k"
    url = convert_url_to_http(before_url)
    assert url == "http://localhost:8000/token/eNwsdW5k"

    # Now with https
    before_url = "wss://localhost:8000/token/1234"
    url = convert_url_to_http(before_url)
    assert url == "https://localhost:8000/token/1234"

    # unsafe url containing ws in token
    before_url = "wss://localhost:8000/token/eNwsdW5k"
    url = convert_url_to_http(before_url)
    assert url == "https://localhost:8000/token/eNwsdW5k"



import pytest
import socketio.exceptions

from zndraw import ZnDraw


@pytest.fixture
def lst():
    return []


@pytest.fixture
def vis(server):
    return ZnDraw(url=server, token="test_token")


@pytest.mark.parametrize("ref", ["lst", "vis"])
def test_zndraw(ref, request):
    """Test the server fixture."""
    obj = request.getfixturevalue(ref)
    assert len(obj) == 0


def test_zndraw_no_connection():
    with pytest.raises(
        socketio.exceptions.ConnectionError,
        match="Could not connect",
    ):
        ZnDraw(url="http://localhost:8080", token="test_token")


@pytest.mark.parametrize("ref", ["lst", "vis"])
def test_extend(ref, s22, request):
    """Test the server fixture."""
    obj = request.getfixturevalue(ref)
    obj.extend(s22)
    assert len(obj) == len(s22)


@pytest.mark.parametrize("ref", ["lst", "vis"])
def test_getitem(ref, s22, request):
    """Test the server fixture."""
    vis = request.getfixturevalue(ref)
    vis.extend(s22)
    assert vis[0] == s22[0]
    assert vis[10] == s22[10]

    with pytest.raises(IndexError):
        vis[100]


@pytest.mark.parametrize("ref", ["lst", "vis"])
def test_insert(ref, s22, water, request):
    """Test the server fixture."""
    vis = request.getfixturevalue(ref)
    vis.extend(s22)
    vis.insert(0, water)
    assert len(vis) == len(s22) + 1
    assert vis[0] == water
    vis.insert(10, water)
    assert len(vis) == len(s22) + 2
    assert vis[10] == water


@pytest.mark.parametrize("ref", ["lst", "vis"])
def test_setitem(ref, s22, water, request):
    vis = request.getfixturevalue(ref)
    vis.extend(s22)
    vis[0] = water
    assert vis[0] == water
    vis[10] = water
    assert vis[10] == water

    assert len(vis) == len(s22)


@pytest.mark.parametrize("ref", ["lst", "vis"])
def test_setitem_slice(ref, s22, water, request):
    vis = request.getfixturevalue(ref)
    vis.extend(s22)
    vis[0:5] = [water] * 5
    assert vis[0:5] == [water] * 5
    assert vis[5:] == s22[5:]


@pytest.mark.parametrize("ref", ["lst", "vis"])
def test_delitem(ref, s22, request):
    vis = request.getfixturevalue(ref)
    vis.extend(s22)
    del vis[0]
    assert len(vis) == len(s22) - 1
    del vis[10]
    assert len(vis) == len(s22) - 2


@pytest.mark.parametrize("ref", ["lst", "vis"])
def test_delitem_slice(ref, server, s22, request):
    vis = request.getfixturevalue(ref)
    vis.extend(s22)
    del vis[0:5]
    assert len(vis) == len(s22) - 5
    del vis[5:10]
    assert len(vis) == len(s22) - 10
    del vis[5:]
    assert len(vis) == 5


@pytest.mark.parametrize("ref", ["lst", "vis"])
def test_append(ref, s22, water, request):
    vis = request.getfixturevalue(ref)
    vis.extend(s22)
    vis.append(water)
    assert len(vis) == len(s22) + 1
    assert vis[-1] == water



import znsocket

from zndraw import ZnDraw


def run_queue(vis, key, msg: dict):
    modifier_queue = znsocket.Dict(
        r=vis.r,
        socket=vis._refresh_client,
        key=f"queue:{vis.token}:{key}",
    )
    modifier_queue.update(msg)
    vis.socket.emit("room:worker:run")
    vis.socket.sleep(7)


def test_run_analysis_distance(server, s22_energy_forces):
    vis = ZnDraw(url=server, token="test_token")
    vis.extend(s22_energy_forces)
    assert vis.figures == {}
    vis.selection = [0, 1]

    run_queue(vis, "analysis", {"Distance": {}})

    fig = vis.figures["Distance"]
    # assert that the x-axis label is "step"
    assert fig.layout.xaxis.title.text == "step"


def test_run_analysis_Properties1D(server, s22_energy_forces):
    vis = ZnDraw(url=server, token="test_token")
    vis.extend(s22_energy_forces)
    assert vis.figures == {}

    run_queue(vis, "analysis", {"Properties1D": {"value": "energy"}})

    fig = vis.figures["Properties1D"]
    # assert that the x-axis label is "step"
    assert fig.layout.xaxis.title.text == "step"
    assert fig.layout.yaxis.title.text == "energy"


def test_run_analysis_Properties2D(server, s22_energy_forces):
    vis = ZnDraw(url=server, token="test_token")
    vis.extend(s22_energy_forces)
    assert vis.figures == {}

    run_queue(
        vis,
        "analysis",
        {"Properties2D": {"x_data": "energy", "y_data": "step", "color": "energy"}},
    )

    fig = vis.figures["Properties2D"]
    # assert that the x-axis label is "step"
    assert fig.layout.yaxis.title.text == "step"
    assert fig.layout.xaxis.title.text == "energy"


def test_run_analysis_DihedralAngle(server, s22_energy_forces):
    vis = ZnDraw(url=server, token="test_token")
    vis.extend(s22_energy_forces)
    assert vis.figures == {}
    vis.selection = [0, 1, 3, 4]

    run_queue(vis, "analysis", {"DihedralAngle": {}})

    fig = vis.figures["DihedralAngle"]
    # assert that the x-axis label is "step"
    assert fig.layout.xaxis.title.text == "step"



from zndraw import ZnDraw


def test_camera(server, s22):
    """Test the server fixture."""
    vis = ZnDraw(url=server, token="test_token")
    vis.extend(s22)

    assert vis.camera["position"] == [5, 5, 5]
    assert vis.camera["target"] == [0, 0, 0]
    assert len(vis.camera) == 2

    vis.camera = {"position": [1, 0, 0], "target": [0, 1, 0]}

    assert vis.camera["position"] == [1, 0, 0]
    assert vis.camera["target"] == [0, 1, 0]



import matplotlib.pyplot as plt

from zndraw import Figure, ZnDraw


def test_figures(server, s22):
    """Test the server fixture."""
    vis = ZnDraw(url=server, token="test_token")
    vis.extend(s22)

    fig, ax = plt.subplots()
    ax.plot([1, 2, 3], [1, 2, 3])

    vis.figures["mtpl"] = Figure(figure=fig)

    assert len(vis.figures) == 1



import pytest

from zndraw import ZnDraw


def test_step(server, s22):
    """Test the server fixture."""
    vis = ZnDraw(url=server, token="test_token")
    vis.extend(s22)

    assert vis.step == 0
    assert isinstance(vis.step, int)

    assert len(vis) == 22

    with pytest.raises(ValueError):
        vis.step = -1

    with pytest.raises(IndexError):
        vis.step = 22

    with pytest.raises(ValueError):
        vis.step = "string"

    vis.step = 5
    assert vis.step == 5
    assert isinstance(vis.step, int)
